from jquery_auto_complete import JQueryAutoComplete
from ajax_m2m_widget import AjaxMultiSelect, auto_complete_view
from dropdown_multi_widget import DropDownMultiple
from raw_id_widget import ModelRawIdWidget
